@@ -6,7 +6,8 @@
 
 module Servant.Client.Core.Internal.Auth where
 
-import           Servant.Client.Core.Internal.Request (Request)
+import           Servant.Client.Core.Internal.Request
+                 (Request)
 
 -- | For a resource protected by authentication (e.g. AuthProtect), we need
 -- to provide the client with some data used to add authentication data
