@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE DataKinds             #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
@@ -12,56 +11,43 @@
 {-# LANGUAGE TypeOperators         #-}
 {-# LANGUAGE UndecidableInstances  #-}
 
-#include "overlapping-compat.h"
 module Servant.Client.Core.Internal.HasClient where
 
-import           Prelude                                ()
+import           Prelude ()
 import           Prelude.Compat
 
-import           Control.Concurrent                     (newMVar, modifyMVar)
-import           Data.Foldable                          (toList)
+import qualified Data.ByteString                        as BS
 import qualified Data.ByteString.Lazy                   as BL
-import           Data.List                              (foldl')
-import           Data.Proxy                             (Proxy (Proxy))
-import           Data.Semigroup                         ((<>))
-import           Data.Sequence                          (fromList)
-import           Data.String                            (fromString)
-import           Data.Text                              (Text, pack)
-import           GHC.TypeLits                           (KnownSymbol, symbolVal)
+import           Data.Foldable
+                 (toList)
+import           Data.List
+                 (foldl')
+import           Data.Proxy
+                 (Proxy (Proxy))
+import           Data.Sequence
+                 (fromList)
+import           Data.String
+                 (fromString)
+import           Data.Text
+                 (Text, pack)
+import           GHC.TypeLits
+                 (KnownSymbol, symbolVal)
 import qualified Network.HTTP.Types                     as H
-import           Servant.API                            ((:<|>) ((:<|>)), (:>),
-                                                         AuthProtect, BasicAuth,
-                                                         BasicAuthData,
-                                                         BuildHeadersTo (..),
-                                                         BuildFromStream (..),
-                                                         ByteStringParser (..),
-                                                         Capture', CaptureAll,
-                                                         Description, EmptyAPI,
-                                                         FramingUnrender (..),
-                                                         Header', Headers (..),
-                                                         HttpVersion, IsSecure,
-                                                         MimeRender (mimeRender),
-                                                         MimeUnrender (mimeUnrender),
-                                                         NoContent (NoContent),
-                                                         QueryFlag, QueryParam',
-                                                         QueryParams, Raw,
-                                                         ReflectMethod (..),
-                                                         RemoteHost, ReqBody',
-                                                         ResultStream(..),
-                                                         SBoolI,
-                                                         Stream,
-                                                         Summary, ToHttpApiData,
-                                                         Vault, Verb,
-                                                         WithNamedContext,
-                                                         contentType,
-                                                         getHeadersHList,
-                                                         getResponse,
-                                                         toQueryParam,
-                                                         toUrlPiece)
-import           Servant.API.ContentTypes               (contentTypes)
-import           Servant.API.Modifiers                  (FoldRequired,
-                                                         RequiredArgument,
-                                                         foldRequiredArgument)
+import           Servant.API
+                 ((:<|>) ((:<|>)), (:>), AuthProtect, BasicAuth, BasicAuthData,
+                 BuildHeadersTo (..), Capture', CaptureAll, Description,
+                 EmptyAPI, FramingUnrender (..), FromSourceIO (..), Header',
+                 Headers (..), HttpVersion, IsSecure, MimeRender (mimeRender),
+                 MimeUnrender (mimeUnrender), NoContent (NoContent), QueryFlag,
+                 QueryParam', QueryParams, Raw, ReflectMethod (..), RemoteHost,
+                 ReqBody', SBoolI, Stream, StreamBody', Summary, ToHttpApiData,
+                 Vault, Verb, WithNamedContext, contentType, getHeadersHList,
+                 getResponse, toQueryParam, toUrlPiece)
+import           Servant.API.ContentTypes
+                 (contentTypes)
+import           Servant.API.Modifiers
+                 (FoldRequired, RequiredArgument, foldRequiredArgument)
+import qualified Servant.Types.SourceT                  as S
 
 import           Servant.Client.Core.Internal.Auth
 import           Servant.Client.Core.Internal.BasicAuth
@@ -216,7 +202,7 @@ instance (KnownSymbol capture, ToHttpApiData a, HasClient m sublayout)
   hoistClientMonad pm _ f cl = \as ->
     hoistClientMonad pm (Proxy :: Proxy sublayout) f (cl as)
 
-instance OVERLAPPABLE_
+instance {-# OVERLAPPABLE #-}
   -- Note [Non-Empty Content Types]
   ( RunClient m, MimeUnrender ct a, ReflectMethod method, cts' ~ (ct ': cts)
   ) => HasClient m (Verb method status cts' a) where
@@ -233,7 +219,7 @@ instance OVERLAPPABLE_
 
   hoistClientMonad _ _ f ma = f ma
 
-instance OVERLAPPING_
+instance {-# OVERLAPPING #-}
   ( RunClient m, ReflectMethod method
   ) => HasClient m (Verb method status cts NoContent) where
   type Client m (Verb method status cts NoContent)
@@ -245,7 +231,7 @@ instance OVERLAPPING_
 
   hoistClientMonad _ _ f ma = f ma
 
-instance OVERLAPPING_
+instance {-# OVERLAPPING #-}
   -- Note [Non-Empty Content Types]
   ( RunClient m, MimeUnrender ct a, BuildHeadersTo ls
   , ReflectMethod method, cts' ~ (ct ': cts)
@@ -268,7 +254,7 @@ instance OVERLAPPING_
 
   hoistClientMonad _ _ f ma = f ma
 
-instance OVERLAPPING_
+instance {-# OVERLAPPING #-}
   ( RunClient m, BuildHeadersTo ls, ReflectMethod method
   ) => HasClient m (Verb method status cts (Headers ls NoContent)) where
   type Client m (Verb method status cts (Headers ls NoContent))
@@ -282,56 +268,25 @@ instance OVERLAPPING_
 
   hoistClientMonad _ _ f ma = f ma
 
-instance OVERLAPPABLE_
-  ( RunClient m, MimeUnrender ct a, ReflectMethod method,
-    FramingUnrender framing a, BuildFromStream a (f a)
-  ) => HasClient m (Stream method status framing ct (f a)) where
+instance {-# OVERLAPPABLE #-}
+  ( RunStreamingClient m, MimeUnrender ct chunk, ReflectMethod method,
+    FramingUnrender framing, FromSourceIO chunk a
+  ) => HasClient m (Stream method status framing ct a) where
 
-  type Client m (Stream method status framing ct (f a)) = m (f a)
-
-  clientWithRoute _pm Proxy req = do
-   sresp <- streamingRequest req
-      { requestAccept = fromList [contentType (Proxy :: Proxy ct)]
-      , requestMethod = reflectMethod (Proxy :: Proxy method)
-      }
-   return . buildFromStream $ ResultStream $ \k ->
-     runStreamingResponse sresp $ \gres -> do
-      let  reader   = responseBody gres
-      let  unrender = unrenderFrames (Proxy :: Proxy framing) (Proxy :: Proxy a)
-           loop bs = do
-             res <- BL.fromStrict <$> reader
-             if BL.null res
-              then return $ parseEOF unrender res
-              else let sofar = (bs <> res)
-                   in case parseIncremental unrender sofar of
-                     Just x -> return x
-                     Nothing -> loop sofar
-      (frameParser, remainder) <- loop BL.empty
-      state <- newMVar remainder
-      let frameLoop bs = do
-              res <- BL.fromStrict <$> reader
-              let addIsEmptyInfo (a, r) = (r, (a, BL.null r && BL.null res))
-              if BL.null res
-                   then if BL.null bs
-                           then return ("", (Right "", True))
-                           else return . addIsEmptyInfo $ parseEOF frameParser bs
-                   else let sofar = (bs <> res)
-                        in case parseIncremental frameParser sofar of
-                          Just x ->  return $ addIsEmptyInfo x
-                          Nothing -> frameLoop sofar
-
-          go = processResult <$> modifyMVar state frameLoop
-          processResult (Right bs,isDone) =
-               if BL.null bs && isDone
-                    then Nothing
-                    else Just $ case mimeUnrender (Proxy :: Proxy ct) bs :: Either String a of
-                                  Left err -> Left err
-                                  Right x -> Right x
-          processResult (Left err, _) = Just (Left err)
-      k go
+  type Client m (Stream method status framing ct a) = m a
 
   hoistClientMonad _ _ f ma = f ma
 
+  clientWithRoute _pm Proxy req = withStreamingRequest req' $ \gres -> do
+      let mimeUnrender'    = mimeUnrender (Proxy :: Proxy ct) :: BL.ByteString -> Either String chunk
+          framingUnrender' = framingUnrender (Proxy :: Proxy framing) mimeUnrender'
+      return $ fromSourceIO $ framingUnrender' $ S.fromAction BS.null (responseBody gres)
+    where
+      req' = req
+          { requestAccept = fromList [contentType (Proxy :: Proxy ct)]
+          , requestMethod = reflectMethod (Proxy :: Proxy method)
+          }
+
 -- | If you use a 'Header' in one of your endpoints in your API,
 -- the corresponding querying function will automatically take
 -- an additional argument of the type specified by your 'Header',
@@ -582,6 +537,18 @@ instance (MimeRender ct a, HasClient m api)
   hoistClientMonad pm _ f cl = \a ->
     hoistClientMonad pm (Proxy :: Proxy api) f (cl a)
 
+instance
+    ( HasClient m api
+    ) => HasClient m (StreamBody' mods framing ctype a :> api)
+  where
+
+    type Client m (StreamBody' mods framing ctype a :> api) = a -> Client m api
+
+    hoistClientMonad pm _ f cl = \a ->
+      hoistClientMonad pm (Proxy :: Proxy api) f (cl a)
+
+    clientWithRoute _pm Proxy _req _body = error "HasClient @StreamBody"
+
 -- | Make the querying function append @path@ to the request path.
 instance (KnownSymbol path, HasClient m api) => HasClient m (path :> api) where
   type Client m (path :> api) = Client m api
