@@ -12,26 +12,36 @@
 
 module Servant.Client.Core.Internal.Request where
 
-import           Prelude                 ()
+import           Prelude ()
 import           Prelude.Compat
 
-import           Control.Monad.Catch     (Exception)
+import           Control.DeepSeq
+                 (NFData (..))
+import           Control.Monad.Catch
+                 (Exception)
 import qualified Data.ByteString         as BS
 import qualified Data.ByteString.Builder as Builder
 import qualified Data.ByteString.Lazy    as LBS
-import           Data.Int                (Int64)
-import           Data.Semigroup          ((<>))
+import           Data.Int
+                 (Int64)
+import           Data.Semigroup
+                 ((<>))
 import qualified Data.Sequence           as Seq
-import           Data.Text               (Text)
-import           Data.Text.Encoding      (encodeUtf8)
-import           Data.Typeable           (Typeable)
-import           GHC.Generics            (Generic)
-import           Network.HTTP.Media      (MediaType)
-import           Network.HTTP.Types      (Header, HeaderName, HttpVersion,
-                                          Method, QueryItem, Status, http11,
-                                          methodGet)
-import           Web.HttpApiData         (ToHttpApiData, toEncodedUrlPiece,
-                                          toHeader)
+import           Data.Text
+                 (Text)
+import           Data.Text.Encoding
+                 (encodeUtf8)
+import           Data.Typeable
+                 (Typeable)
+import           GHC.Generics
+                 (Generic)
+import           Network.HTTP.Media
+                 (MediaType, mainType, parameters, subType)
+import           Network.HTTP.Types
+                 (Header, HeaderName, HttpVersion (..), Method, QueryItem,
+                 Status (..), http11, methodGet)
+import           Servant.API
+                 (ToHttpApiData, toEncodedUrlPiece, toHeader)
 
 -- | A type representing possible errors in a request
 --
@@ -51,6 +61,20 @@ data ServantError =
 
 instance Exception ServantError
 
+instance NFData ServantError where
+    rnf (FailureResponse res)            = rnf res
+    rnf (DecodeFailure err res)          = rnf err `seq` rnf res
+    rnf (UnsupportedContentType mt' res) =
+        mediaTypeRnf mt' `seq`
+        rnf res
+      where
+        mediaTypeRnf mt =
+            rnf (mainType mt) `seq`
+            rnf (subType mt) `seq`
+            rnf (parameters mt)
+    rnf (InvalidContentTypeHeader res)   = rnf res
+    rnf (ConnectionError err)            = rnf err
+
 data RequestF a = Request
   { requestPath        :: a
   , requestQueryString :: Seq.Seq QueryItem
@@ -80,8 +104,18 @@ data GenResponse a = Response
   , responseBody        :: a
   } deriving (Eq, Show, Generic, Typeable, Functor, Foldable, Traversable)
 
+instance NFData a => NFData (GenResponse a) where
+    rnf (Response sc hs hv body) =
+        rnfStatus sc `seq`
+        rnf hs `seq`
+        rnfHttpVersion hv `seq`
+        rnf body
+      where
+        rnfStatus (Status code msg) = rnf code `seq` rnf msg
+        rnfHttpVersion (HttpVersion _ _) = () -- HttpVersion fields are strict
+
 type Response = GenResponse LBS.ByteString
-newtype StreamingResponse = StreamingResponse { runStreamingResponse :: forall a. (GenResponse (IO BS.ByteString) -> IO a) -> IO a }
+type StreamingResponse = GenResponse (IO BS.ByteString)
 
 -- A GET request to the top-level path
 defaultRequest :: Request
