@@ -6,31 +6,36 @@
 -- | Types for possible backends to run client-side `Request` queries
 module Servant.Client.Core.Internal.RunClient where
 
-import           Prelude                              ()
+import           Prelude ()
 import           Prelude.Compat
 
-import           Control.Monad                        (unless)
-import           Control.Monad.Free                   (Free (..), liftF)
-import           Data.Foldable                        (toList)
-import           Data.Proxy                           (Proxy)
+import           Control.Monad
+                 (unless)
+import           Control.Monad.Free
+                 (Free (..), liftF)
+import           Data.Foldable
+                 (toList)
+import           Data.Proxy
+                 (Proxy)
 import qualified Data.Text                            as T
-import           Network.HTTP.Media                   (MediaType, matches,
-                                                       parseAccept, (//))
-import           Servant.API                          (MimeUnrender,
-                                                       contentTypes,
-                                                       mimeUnrender)
+import           Network.HTTP.Media
+                 (MediaType, matches, parseAccept, (//))
+import           Servant.API
+                 (MimeUnrender, contentTypes, mimeUnrender)
 
-import           Servant.Client.Core.Internal.Request (Request, Response, GenResponse (..),
-                                                       StreamingResponse (..),
-                                                       ServantError (..))
 import           Servant.Client.Core.Internal.ClientF
+import           Servant.Client.Core.Internal.Request
+                 (GenResponse (..), Request, Response, ServantError (..),
+                 StreamingResponse)
 
 class Monad m => RunClient m where
   -- | How to make a request.
   runRequest :: Request -> m Response
-  streamingRequest :: Request -> m StreamingResponse
   throwServantError :: ServantError -> m a
 
+class RunClient m =>  RunStreamingClient m where
+    withStreamingRequest :: Request -> (StreamingResponse -> IO a) ->  m a
+
 checkContentTypeHeader :: RunClient m => Response -> m MediaType
 checkContentTypeHeader response =
   case lookup "Content-Type" $ toList $ responseHeaders response of
@@ -53,5 +58,10 @@ decodedAs response contentType = do
 
 instance ClientF ~ f => RunClient (Free f) where
     runRequest req  = liftF (RunRequest req id)
-    streamingRequest req = liftF (StreamingRequest req id)
     throwServantError = liftF . Throw
+
+{-
+Free and streaming?
+instance ClientF ~ f => RunStreamingClient (Free f) where
+    streamingRequest req = liftF (StreamingRequest req id)
+-}
